!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
module LIS_DAobservationsMod

!BOP
!
! !MODULE: LIS_DAobservationsMod
!
! !DESCRIPTION:
!  The code in this file controls the handling of 
!  observations to be used for data assimilation. 
!   
! !REVISION HISTORY:
!
!  21 Jun 2006: Sujay Kumar; Initial implementation
!
  use ESMF

  implicit none

  PRIVATE
!------------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!------------------------------------------------------------------------------
  public ::  LIS_initDAobservations
  public ::  LIS_readDAobservations
  public ::  LIS_perturb_DAobservations
!------------------------------------------------------------------------------
! !PUBLIC TYPES:
!------------------------------------------------------------------------------
  public :: LIS_OBS_State
  public :: LIS_OBS_Pert_State

  type(ESMF_State), allocatable :: LIS_OBS_State(:,:)
  type(ESMF_State), allocatable :: LIS_OBS_Pert_State(:,:)

!EOP

contains

!BOP
! 
! !ROUTINE: LIS_initDAobservations
! \label{LIS_initDAobservations}
! 
! !INTERFACE: 
  subroutine LIS_initDAobservations
! 
! !DESCRIPTION: 
!  This routine allocates the structures required for managing 
!  observational data used for data assimilation. The routine
!  creates ESMF State objects used to incorporate 
!  observational data. 
! 
!  The methods invoked are: 
!  \begin{description}
!  \item[LIS\_DAobs\_plugin](\ref{LIS_DAobs_plugin}) \newline
!    sets up function table registries for implemented 
!    observation sources to be assimilated.
!  \item[readObsDataConfig](\ref{readobsdataconfig}) \newline
!    invokes the generic method in the registry to 
!    set up the structures to read the observation data, for 
!    assimilation.
!  \item[perturbinit](\ref{perturbinit}) \newline
!    invokes the generic method in the registry to initialize
!    the observation perturbation algorithm.
!  \item[perturbsetup](\ref{perturbsetup}) \newline
!    invokes the generic method in the registry to populate
!    the observation perturbation objects. 
!  \end{description}
!
! !USES:     
    use LIS_coreMod, only : LIS_rc
    use LIS_logMod,  only : LIS_logunit, LIS_verify
    use LIS_DAobs_pluginMod, only : LIS_DAobs_plugin
!EOP
    integer       :: n 
    integer       :: status
    character*1   :: nestid(2)
    character*1   :: caseid(3)
    character*100 :: temp
    integer              :: max_index
    integer              :: i, k
    integer,    allocatable  :: insts(:)
    logical              :: name_found
    character*20         :: alglist(10)

    call LIS_DAobs_plugin       

    if(LIS_rc%ndas.gt.0) then 

       allocate(LIS_OBS_State(LIS_rc%nnest, LIS_rc%ndas))
       allocate(LIS_OBS_Pert_State(LIS_rc%nnest, LIS_rc%ndas))

       do n=1,LIS_rc%nnest
          do k=1,LIS_rc%nperts
             write(unit=temp,fmt='(i2.2)') n
             read(unit=temp,fmt='(2a1)') nestid

             write(unit=temp,fmt='(i3.3)') k
             read(unit=temp,fmt='(3a1)') caseid
          
             LIS_OBS_State(n,k) = ESMF_StateCreate(name=&
                  "Observations State"//nestid(1)//nestid(2) &
                  //"_"//caseid(1)//caseid(2)//caseid(3), rc=status)
             call LIS_verify(status, 'obs state create')

          
             LIS_OBS_Pert_State(n,k) = ESMF_StateCreate(&
                  name="Observations Perturbations"//nestid(1)//nestid(2)//&
                  "_"//caseid(1)//caseid(2)//caseid(3), rc=status)
             call LIS_verify(status, 'obs pert state create')

          enddo
       enddo
       write(LIS_logunit,*) 'Successfully created observations state'
       do k=1, LIS_rc%ndas
          call readObsDataConfig(trim(LIS_rc%daset(k))//char(0),k,&
               LIS_OBS_State(:,k), LIS_OBS_Pert_State(:,k))
       enddo

       max_index = -1
       do i=1,LIS_rc%nperts
          if(max_index.eq.-1.and.LIS_rc%perturb_obs(i).ne."none") then 
             max_index = 1
             alglist(max_index) = LIS_rc%perturb_obs(i)
          else
             name_found = .false. 
             do k=1,max_index
                if(LIS_rc%perturb_obs(i).ne."none".and.&
                     LIS_rc%perturb_obs(i).eq.alglist(k)) then
                   name_found = .true. 
                endif
             enddo
             if(.not.name_found.and.max_index.ne.-1) then 
                max_index = max_index + 1
                alglist(max_index) = LIS_rc%perturb_obs(i)
             endif
          endif
       enddo
       
       if(max_index.gt.0) then 
          do i=1,max_index
!Call this only once for all instances of the algorithm
             call perturbinit(trim(alglist(i))//char(0), 3)
          enddo
       endif
       
       do i=1,LIS_rc%nperts
          if(LIS_rc%perturb_obs(i).ne."none") then 
             call perturbsetup(trim(LIS_rc%perturb_obs(i))//char(0), 3, i,&
                  LIS_OBS_State(:,i), LIS_OBS_Pert_State(:,i))
          endif
       enddo
    endif

  end subroutine LIS_initDAobservations

!BOP
! 
! !ROUTINE: LIS_readDAobservations
! \label{LIS_readDAobservations}
! 
! !INTERFACE: 
  subroutine LIS_readDAobservations(n)
! !USES: 
    use LIS_coreMod,   only : LIS_rc

! !ARGUMENTS: 
    integer, intent(in) :: n

! 
! !DESCRIPTION: 
!  This routines calls the method to read the specific observation
!  source. 
! 
!  The methods invoked are: 
!  \begin{description}
!  \item[readDAObs](\ref{readdaobs}) \newline
!   invokes the method in the registry to read the observations and 
!   incorporate into the observation state.
!  \end{description}
!EOP
    integer          :: k 

    do k=1,LIS_rc%ndas
       call readDAObs(trim(LIS_rc%daset(k))//char(0), n, &
            LIS_OBS_State(n,k), LIS_OBS_Pert_State(n,k))
    enddo

  end subroutine LIS_readDAobservations

!BOP
! 
! !ROUTINE: LIS_perturb_DAobservations
! \label{LIS_perturb_DAobservations}
!
! !INTERFACE: 
  subroutine LIS_perturb_DAobservations(n)
! !USES: 
    use LIS_coreMod,   only : LIS_rc
    use LIS_logMod,    only : LIS_verify

! !ARGUMENTS: 
    integer, intent(in) :: n
! 
! !DESCRIPTION: 
!  This routines calls the method to perturb the observations based on the
!  specified choice. 
! 
!  The methods invoked are: 
!  \begin{description}
!  \item[perturbmethod](\ref{perturbmethod}) \newline
!   invokes the specified algorithm to perturb the observations. 
!  \item[writeDAObs](\ref{writedaobs}) \newline
!   invokes the registry method to write the perturbed and 
!   processed observations to disk. 
!  \end{description}
!EOP
    real                    :: curr_time
    logical                 :: data_status
    integer                 :: status
    integer                 :: k

    do k=1,LIS_rc%ndas
       if(LIS_rc%perturb_obs(k).ne."none") then 
          curr_time = float(LIS_rc%hr)*3600+60*float(LIS_rc%mn)+float(LIS_rc%ss)
!          if(mod(curr_time,real(LIS_rc%pertobsInterval(k))).eq.0)then
             call ESMF_AttributeGet(LIS_OBS_State(n,k),"Data Update Status",&
                  data_status,rc=status)
             call LIS_verify(status, &
                  "ESMF_AttributeGet: Data Update Status failed in LIS_perturb_DAobservations")
             if(data_status) then
                call perturbmethod(trim(LIS_rc%perturb_obs(k))//char(0),3,n,k, &
                     LIS_OBS_State(n,k), LIS_OBS_Pert_State(n,k))
             endif
!          endif
          endif
       if(LIS_rc%wobs(k).gt.0) then 
          call writeDAObs(trim(LIS_rc%daset(k))//char(0), n, LIS_OBS_State(n,k))
       endif
    enddo         
    
  end subroutine LIS_perturb_DAobservations

end module LIS_DAobservationsMod
