!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
module LIS_DAobs_pluginMod
!BOP
!
! !MODULE: LIS_DAobs_pluginMod
! 
! !DESCRIPTION: 
!   This module contains the definition of the functions that are
!   used to read observation data for data assimiliation.  
!   The user defined functions are incorporated into 
!   the appropriate registry to be later invoked through generic calls. 
!   
! !REVISION HISTORY: 
!  27 Feb 2005;   Sujay Kumar  Initial Specification
! 
!EOP  
  implicit none
  PRIVATE
!------------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!------------------------------------------------------------------------------
  PUBLIC :: LIS_DAobs_plugin
  
contains
!BOP
! !ROUTINE: LIS_DAobs_plugin
!  \label{LIS_DAobs_plugin}
!
! !DESCRIPTION:
!
!  This is a plugin point for introducing routines to handle the 
!  observation data for assimilation.As explaind in 
!  the {\tt dataassim\_module}, there are three
!  different abstractions associated with the 
!  data assimilation implementation. The implementations defined in this
!  subroutine complete the "wirings" required to complete the 
!  observation data-related abstractions. Other required
!  interfaces defined in {\tt lsmda\_pluginMod} and {\tt dataassim\_pluginMod}
!  should be completed for a successful implementation. 
! 
!  The following interfaces should be implemented in this routine. 
!  \begin{description}
!  \item[Setup]
!      Initialization of data and memory structures
!      (to be registered using {\tt registerreaddaobssetup} and later called 
!       through {\tt daobssetup})
!  \item[read observations]
!      Routines to read the observation data and perform any 
!      spatial transformation. 
!      (to be registered using {\tt registerreaddaobs} and later called
!       through {\tt readobservations})
!  \item[get number of selected observations]
!      routines to retrieve the number of selected observations for 
!      the selected modeling point. 
!      (to be registered using {\tt registergetnso} and later called
!      through {\tt getselctedobsnumber}
!   \end{description}
! 
!  The user-defined functions are included in the registry using a single 
!  index. For example, consider an instance where soil moisture
!  assimilation using TMI soil moisture data is conducted. The methods
!  should be defined in the registry as follows, if the index of the 
!  'assimilation set' (assimilating TMI to update soil moisture variables)
!  is defined to be 1
!
!  \begin{verbatim}
!    call registerdaobssetup(1,TMIsmobs_setup)  
!    call registerreaddaobs(1,read_TMIsm)
!    call registergetnso(1,getNSO_TMIsm)
!  \end{verbatim}
!
!   The functions registered above are invoked using generic calls as 
!   follows: 
!  
!  \begin{verbatim}
!    call daobssetup(1)       -  calls TMIsmobs_setup
!    call readobservations(1) -  calls read_TMIsm
!  \end{verbatim}
!
!   In the LIS code, the above calls are typically invoked in the 
!   following manner. 
!   \begin{verbatim}
!    call daobssetup(lis%daset, )       
!    call readobservations(lis%daset)
!   \end{verbatim}
!   where $lis\%daset$ is set through the configuration
!   utility, enabling the user to make selections at runtime. 
!
! !INTERFACE:
  subroutine LIS_DAobs_plugin
!EOP
    use LIS_pluginIndices
#if 0
    use syntheticsmobs_module,   only : syntheticsmobs_setup
    use syntheticsweobs_module,  only : syntheticsweobs_setup
    use syntheticlstobs_module,  only : syntheticlstobs_setup
    use multisynsmobs_Mod,       only : multisynsmobs_setup
    use ISCCP_Tskin_module,      only : ISCCP_Tskin_setup
#endif
    use MODISscaobs_module,      only : MODISscaobs_setup
#if 0
    use SNODEPobs_Mod,           only : SNODEPobs_setup
#endif
    use NASA_AMSREsm_Mod,        only : NASA_AMSREsm_setup
    use LPRM_AMSREsm_Mod,        only : LPRM_AMSREsm_setup
    use ESACCI_sm_Mod,           only : ESACCI_sm_setup
#if 0
    use ANSASWEsnow_Mod,         only : ANSASWEsnow_setup
#endif
    use ANSASCFsnow_Mod,         only : ANSASCFsnow_setup
#if 0
    use ANSASNWDsnow_Mod,        only : ANSASNWDsnow_setup
    use AMSRE_SWE_Mod,           only : AMSRE_SWE_setup
!    use AMSRE_snow_Mod,          only : AMSRE_snow_setup !yliu
#endif
    use PMW_snow_Mod,            only : PMW_snow_setup !yliu
#if 0
    use SMMRSNWDsnow_Mod,        only : SMMRSNWDsnow_setup
    use SSMISNWDsnow_Mod,        only : SSMISNWDsnow_setup
#endif
    use WindSatsm_Mod,           only : WindSatsm_setup
#if 0
    use WindSatCsm_Mod,          only : WindSatCsm_setup
#endif
    use GRACEobs_module,         only : GRACEobs_setup
#if 0
    use simGRACEJPLobs_module,   only : simGRACEJPLobs_setup
    use SYN_LBAND_TB_Mod,        only : SYN_LBAND_TB_setup
#endif
    use SMOPSsm_Mod,             only : SMOPSsm_setup   
#if 0
    use ASCAT_TUWsm_Mod,         only : ASCAT_TUWsm_setup
    use GCOMW_AMSR2L3sm_Mod,     only : GCOMW_AMSR2L3sm_setup
    use SMOSL2sm_Mod,            only : SMOSL2sm_setup
#endif

#if 0
    external read_syntheticsmobs, getNSO_synsm, write_syntheticsmobs
    external read_syntheticsweobs,getNSO_synswe
    external read_syntheticlstobs,getNSO_synlst
    external read_multisynsmobs, getNSO_multisynsm
    external read_ISCCP_Tskin, write_ISCCP_Tskin, getNSO_ISCCP_Tskin
#endif
    external read_MODISscaobs, getNSO_MODISsca, write_MODISsca
#if 0
    external read_SNODEPobs, write_SNODEPobs, getNSO_SNODEP
#endif
    external read_NASA_AMSREsm, write_NASA_AMSREsmobs, getNSO_NASA_AMSREsm
    external read_LPRM_AMSREsm, write_LPRM_AMSREsmobs, getNSO_LPRM_AMSREsm
    external read_ESACCIsm, write_ESACCIsmobs, getNSO_ESACCIsm
#if 0
    external read_ANSASWEsnow, write_ANSASWEsnowobs, getNSO_ANSASWEsnow
#endif
    external read_ANSASCFsnow, write_ANSASCFsnowobs, getNSO_ANSASCFsnow
#if 0
    external read_ANSASNWDsnow, write_ANSASNWDsnowobs, getNSO_ANSASNWDsnow
    external read_SMMRSNWDsnow, write_SMMRSNWDsnowobs, getNSO_SMMRSNWDsnow
    external read_SSMISNWDsnow, write_SSMISNWDsnowobs, getNSO_SSMISNWDsnow
#endif
    external read_WindSatsm, write_WindSatsmobs, getNSO_WindSatsm
#if 0
    external read_WindSatCsm, write_WindSatCsmobs, getNSO_WindSatCsm
    external read_AMSRE_SWE, write_AMSRE_SWEobs, getNSO_AMSRE_SWE
!    external read_AMSRE_snow, write_AMSRE_snowobs, getNSO_AMSRE_snow !yliu
#endif
    external read_PMW_snow, write_PMW_snowobs, getNSO_PMW_snow !yliu
    external read_GRACEobs, getNSO_GRACE, write_GRACEobs    
#if 0
    external read_simGRACEJPLobs, getNSO_simGRACEJPL, write_simGRACEJPLobs    
    external read_SYN_LBAND_TB, getNSO_SYN_LBAND_TB, write_SYN_LBAND_TB
#endif
    external read_SMOPSsm, getNSO_SMOPSsm, write_SMOPSsmobs
#if 0
    external read_ASCAT_TUWsm, getNSO_ASCAT_TUWsm, write_ASCAT_TUWsmobs
    external read_GCOMW_AMSR2L3sm, getNSO_GCOMW_AMSR2L3,  write_GCOMW_AMSR2L3obs
    external read_SMOSL2sm,getNSO_SMOSL2sm,write_SMOSL2smobs
#endif

#if 0
!synthetic noah soil moisture    
    call registerdaobssetup(trim(LIS_synsmId)//char(0),syntheticsmobs_setup)
    call registerreaddaobs(trim(LIS_synsmId)//char(0),read_syntheticsmobs)
    call registerwritedaobs(trim(LIS_synsmId)//char(0),write_syntheticsmobs)
    call registergetnso(trim(LIS_synsmId)//char(0),getNSO_synsm)

!synthetic noah SWE1
    call registerdaobssetup(trim(LIS_synsweId)//char(0),syntheticsweobs_setup)
    call registerreaddaobs(trim(LIS_synsweId)//char(0),read_syntheticsweobs)
    call registergetnso(trim(LIS_synsweId)//char(0),getNSO_synswe)
!synthetic lst
    call registerdaobssetup(trim(LIS_synlstId)//char(0),syntheticlstobs_setup)
    call registerreaddaobs(trim(LIS_synlstId)//char(0),read_syntheticlstobs)
    call registergetnso(trim(LIS_synlstId)//char(0),getNSO_synlst)

!multi layer synthetic sm obs 
    call registerdaobssetup(trim(LIS_multisynsmobsId)//char(0),multisynsmobs_setup)
    call registerreaddaobs(trim(LIS_multisynsmobsId)//char(0),read_multisynsmobs)
    call registergetnso(trim(LIS_multisynsmobsId)//char(0),getNSO_multisynsm)


!ISSP Tskin 
    call registerdaobssetup(trim(LIS_isccpTskinId)//char(0),ISCCP_Tskin_setup)
    call registerreaddaobs(trim(LIS_isccpTskinId)//char(0),read_ISCCP_Tskin)
    call registerwritedaobs(trim(LIS_isccpTskinId)//char(0),write_ISCCP_Tskin)
    call registergetnso(trim(LIS_isccpTskinId)//char(0),getNSO_ISCCP_Tskin)
#endif

!MODIS snow cover fraction
    call registerdaobssetup(trim(LIS_modisscfId)//char(0),MODISscaobs_setup)
    call registerreaddaobs(trim(LIS_modisscfId)//char(0),read_MODISscaobs)
    call registerwritedaobs(trim(LIS_modisscfId)//char(0),write_MODISsca)
    call registergetnso(trim(LIS_modisscfId)//char(0),getNSO_MODISsca)

#if 0
!SNODEP obs 
    call registerdaobssetup(trim(LIS_snodepobsId)//char(0),SNODEPobs_setup)
    call registerreaddaobs(trim(LIS_snodepobsId)//char(0),read_SNODEPobs)
    call registerwritedaobs(trim(LIS_snodepobsId)//char(0),write_SNODEPobs)
    call registergetnso(trim(LIS_snodepobsId)//char(0),getNSO_SNODEP)
#endif

!NASA AMSRE obs 
    call registerdaobssetup(trim(LIS_NASA_AMSREsmobsId)//char(0),NASA_AMSREsm_setup)
    call registerreaddaobs(trim(LIS_NASA_AMSREsmobsId)//char(0),read_NASA_AMSREsm)
    call registerwritedaobs(trim(LIS_NASA_AMSREsmobsId)//char(0),write_NASA_AMSREsmobs)
    call registergetnso(trim(LIS_NASA_AMSREsmobsId)//char(0),getNSO_NASA_AMSREsm)

!LPRM AMSRE obs 
    call registerdaobssetup(trim(LIS_LPRM_AMSREsmobsId)//char(0),LPRM_AMSREsm_setup)
    call registerreaddaobs(trim(LIS_LPRM_AMSREsmobsId)//char(0),read_LPRM_AMSREsm)
    call registerwritedaobs(trim(LIS_LPRM_AMSREsmobsId)//char(0),write_LPRM_AMSREsmobs)
    call registergetnso(trim(LIS_LPRM_AMSREsmobsId)//char(0),getNSO_LPRM_AMSREsm)

!ESACCI sm obs
    call registerdaobssetup(trim(LIS_ESACCIsmobsId)//char(0),ESACCI_sm_setup)
    call registerreaddaobs(trim(LIS_ESACCIsmobsId)//char(0),read_ESACCIsm)
    call registerwritedaobs(trim(LIS_ESACCIsmobsId)//char(0),write_ESACCIsmobs)
    call registergetnso(trim(LIS_ESACCIsmobsId)//char(0),getNSO_ESACCIsm)


#if 0
!ANSA SWE snow obs 
    call registerdaobssetup(trim(LIS_ANSASWEsnowobsId)//char(0),ANSASWEsnow_setup)
    call registerreaddaobs(trim(LIS_ANSASWEsnowobsId)//char(0),read_ANSASWEsnow)
    call registerwritedaobs(trim(LIS_ANSASWEsnowobsId)//char(0),write_ANSASWEsnowobs)
    call registergetnso(trim(LIS_ANSASWEsnowobsId)//char(0),getNSO_ANSASWEsnow)
#endif

!ANSA SCF snow obs 
    call registerdaobssetup(trim(LIS_ANSASCFsnowobsId)//char(0),ANSASCFsnow_setup)
    call registerreaddaobs(trim(LIS_ANSASCFsnowobsId)//char(0),read_ANSASCFsnow)
    call registerwritedaobs(trim(LIS_ANSASCFsnowobsId)//char(0),write_ANSASCFsnowobs)
    call registergetnso(trim(LIS_ANSASCFsnowobsId)//char(0),getNSO_ANSASCFsnow)

#if 0
!ANSA SNWD snow obs 
    call registerdaobssetup(trim(LIS_ANSASNWDsnowobsId)//char(0),&
         ANSASNWDsnow_setup)
    call registerreaddaobs(trim(LIS_ANSASNWDsnowobsId)//char(0),&
         read_ANSASNWDsnow)
    call registerwritedaobs(trim(LIS_ANSASNWDsnowobsId)//char(0),&
         write_ANSASNWDsnowobs)
    call registergetnso(trim(LIS_ANSASNWDsnowobsId)//char(0),&
         getNSO_ANSASNWDsnow)
#endif

#if 0
!SMMR SNWD snow obs 
    call registerdaobssetup(trim(LIS_SMMRSNWDsnowobsId)//char(0),&
         SMMRSNWDsnow_setup)
    call registerreaddaobs(trim(LIS_SMMRSNWDsnowobsId)//char(0),&
         read_SMMRSNWDsnow)
    call registerwritedaobs(trim(LIS_SMMRSNWDsnowobsId)//char(0),&
         write_SMMRSNWDsnowobs)
    call registergetnso(trim(LIS_SMMRSNWDsnowobsId)//char(0),&
         getNSO_SMMRSNWDsnow)
#endif

#if 0
!SSMI SNWD snow obs 
    call registerdaobssetup(trim(LIS_SSMISNWDsnowobsId)//char(0),&
         SSMISNWDsnow_setup)
    call registerreaddaobs(trim(LIS_SSMISNWDsnowobsId)//char(0),&
         read_SSMISNWDsnow)
    call registerwritedaobs(trim(LIS_SSMISNWDsnowobsId)//char(0),&
         write_SSMISNWDsnowobs)
    call registergetnso(trim(LIS_SSMISNWDsnowobsId)//char(0),&
         getNSO_SSMISNWDsnow)
#endif

#if 0
!AMSRE SWE
    call registerdaobssetup(trim(LIS_AMSREsweobsId)//char(0),AMSRE_SWE_setup)
    call registerreaddaobs(trim(LIS_AMSREsweobsId)//char(0),read_AMSRE_SWE)
    call registerwritedaobs(trim(LIS_AMSREsweobsId)//char(0),write_AMSRE_SWEobs)
    call registergetnso(trim(LIS_AMSREsweobsId)//char(0),getNSO_AMSRE_SWE)
#endif

!AMSRE Snow Depth or SWE, yliu
!    call registerdaobssetup(trim(LIS_AMSREsnowobsId)//char(0),AMSRE_snow_setup) !yliu
!    call registerreaddaobs(trim(LIS_AMSREsnowobsId)//char(0),read_AMSRE_snow)   !yliu
!    call registerwritedaobs(trim(LIS_AMSREsnowobsId)//char(0),write_AMSRE_snowobs) !yliu
!    call registergetnso(trim(LIS_AMSREsnowobsId)//char(0),getNSO_AMSRE_snow)   !yliu

!PMW Snow Depth or SWE, yliu
    call registerdaobssetup(trim(LIS_PMWsnowobsId)//char(0),PMW_snow_setup) !yliu
    call registerreaddaobs(trim(LIS_PMWsnowobsId)//char(0),read_PMW_snow)   !yliu
    call registerwritedaobs(trim(LIS_PMWsnowobsId)//char(0),write_PMW_snowobs) !yliu
    call registergetnso(trim(LIS_PMWsnowobsId)//char(0),getNSO_PMW_snow)   !yliu

!WindSat soil moisture obs 
    call registerdaobssetup(trim(LIS_WindSatsmobsId)//char(0),WindSatsm_setup)
    call registerreaddaobs(trim(LIS_WindSatsmobsId)//char(0),read_WindSatsm)
    call registerwritedaobs(trim(LIS_WindSatsmobsId)//char(0),write_WindSatsmobs)
    call registergetnso(trim(LIS_WindSatsmobsId)//char(0),getNSO_WindSatsm)

#if 0
!WindSat Cband (over Toulouse only) soil moisture obs 
    call registerdaobssetup(trim(LIS_WindSatCsmobsId)//char(0),WindSatCsm_setup)
    call registerreaddaobs(trim(LIS_WindSatCsmobsId)//char(0),read_WindSatCsm)
    call registerwritedaobs(trim(LIS_WindSatCsmobsId)//char(0),write_WindSatCsmobs)
    call registergetnso(trim(LIS_WindSatCsmobsId)//char(0),getNSO_WindSatCsm)
#endif

! GRACE TWS obs 
    call registerdaobssetup(trim(LIS_GRACEtwsobsId)//char(0),GRACEobs_setup)
    call registerreaddaobs(trim(LIS_GRACEtwsobsId)//char(0),read_GRACEobs)
    call registerwritedaobs(trim(LIS_GRACEtwsobsId)//char(0),write_GRACEobs)
    call registergetnso(trim(LIS_GRACEtwsobsId)//char(0),getNSO_GRACE)
#if 0

!simulated GRACE
    call registerdaobssetup(trim(LIS_simGRACEJPLobsId)//char(0),&
         simGRACEJPLobs_setup)
    call registerreaddaobs(trim(LIS_simGRACEJPLobsId)//char(0),&
         read_simGRACEJPLobs)
    call registerwritedaobs(trim(LIS_simGRACEJPLobsId)//char(0),&
         write_simGRACEJPLobs)
    call registergetnso(trim(LIS_simGRACEJPLobsId)//char(0),&
         getNSO_simGRACEJPL)

!Synthetic L-band Tb
    call registerdaobssetup(trim(LIS_synLbandTbobsId)//char(0),SYN_LBAND_TB_setup)
    call registerreaddaobs(trim(LIS_synLbandTbobsId)//char(0),read_SYN_LBAND_TB)
    call registerwritedaobs(trim(LIS_synLbandTbobsId)//char(0),write_SYN_LBAND_TB)
    call registergetnso(trim(LIS_synLbandTbobsId)//char(0),getNSO_SYN_LBAND_TB)
#endif

    call registerdaobssetup(trim(LIS_SMOPSsmobsId)//char(0),SMOPSsm_setup)
    call registerreaddaobs(trim(LIS_SMOPSsmobsId)//char(0),read_SMOPSsm)
    call registerwritedaobs(trim(LIS_SMOPSsmobsId)//char(0),write_SMOPSsmobs)
    call registergetnso(trim(LIS_SMOPSsmobsId)//char(0),getNSO_SMOPSsm)

#if 0
    call registerdaobssetup(trim(LIS_ASCAT_TUWsmobsId)//char(0),ASCAT_TUWsm_setup)
    call registerreaddaobs(trim(LIS_ASCAT_TUWsmobsId)//char(0),read_ASCAT_TUWsm)
    call registerwritedaobs(trim(LIS_ASCAT_TUWsmobsId)//char(0),write_ASCAT_TUWsmobs)
    call registergetnso(trim(LIS_ASCAT_TUWsmobsId)//char(0),getNSO_ASCAT_TUWsm)
#endif

#if 0
    call registerdaobssetup(trim(LIS_GCOMW_AMSR2L3smobsId)//char(0),GCOMW_AMSR2L3sm_setup)
    call registerreaddaobs(trim(LIS_GCOMW_AMSR2L3smobsId)//char(0),read_GCOMW_AMSR2L3sm)
    call registerwritedaobs(trim(LIS_GCOMW_AMSR2L3smobsId)//char(0),write_GCOMW_AMSR2L3obs)
    call registergetnso(trim(LIS_GCOMW_AMSR2L3smobsId)//char(0),getNSO_GCOMW_AMSR2L3)
#endif

#if 0
    call registerdaobssetup(trim(LIS_SMOSL2smobsId)//char(0),SMOSL2sm_setup)
    call registerreaddaobs(trim(LIS_SMOSL2smobsId)//char(0),read_SMOSL2sm)
    call registerwritedaobs(trim(LIS_SMOSL2smobsId)//char(0),write_SMOSL2smobs)
    call registergetnso(trim(LIS_SMOSL2smobsId)//char(0),getNSO_SMOSL2sm)
#endif

  end subroutine LIS_DAobs_plugin
end module LIS_DAobs_pluginMod
